<?php

include_once "AbstractManufacturer.php";

/**
 * Class ExpressionManufacturer
 *
 * Date: 25.03.13
 * Time: 15:12
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ExpressionManufacturer extends AbstractManufacturer
{

    /**
     * A Singleton Instance of the ExpressionManufacturer
     *
     * @see ExpressionManufacturer::_getInstance()
     * @see ExpressionManufacturer::__construct()
     *
     * @var null|ExpressionManufacturer
     */
    protected static $instance = null;

    /**
     * Get an Instance of ExpressionManufacturer
     *
     * @return ExpressionManufacturer
     */
    public static function _getInstance()
    {
        if (self::$instance == null) {
            self::$instance = new ExpressionManufacturer();
        }
        return self::$instance;
    }

    /**
     * /**
     * Create a new specific Composite object which is representing the delivered Attribute
     *
     * @param string $attribute
     *
     * @return AbstractComposite|AnswerOfQuestionExpression|NumberOfResultExpression|NumericResultExpression|PercentageResultExpression|ResultOfAnswerOfQuestionExpression|StringResultExpression
     * @throws UnsupportedExpression
     */
    public function manufacture($attribute)
    {
        $expression = null;

        switch (true) {
            case preg_match(ResultOfAnswerOfQuestionExpression::$pattern, $attribute):
                $expression = new ResultOfAnswerOfQuestionExpression();
                break;
            case preg_match(AnswerOfQuestionExpression::$pattern, $attribute):
                $expression = new AnswerOfQuestionExpression();
                break;
            case preg_match(PercentageResultExpression::$pattern, $attribute):
                $expression = new PercentageResultExpression();
                break;
            case preg_match(NumberOfResultExpression::$pattern, $attribute):
                $expression = new NumberOfResultExpression();
                break;
            case preg_match(NumericResultExpression::$pattern, $attribute):
                $expression = new NumericResultExpression();
                break;
            case preg_match(StringResultExpression::$pattern, $attribute):
                $expression = new StringResultExpression();
                break;
            case preg_match(MatchingResultExpression::$pattern, $attribute):
                $expression = new MatchingResultExpression();
                break;
            case preg_match(OrderingResultExpression::$pattern, $attribute):
                $expression = new OrderingResultExpression();
                break;
            case preg_match(ExclusiveResultExpression::$pattern, $attribute):
                $expression = new ExclusiveResultExpression();
                break;
            case preg_match(EmptyAnswerExpression::$pattern, $attribute):
                $expression = new EmptyAnswerExpression();
                break;
            default:
                throw new UnsupportedExpression($attribute);
                break;
        }

        $expression->parseValue($attribute);
        return $expression;
    }

    /**
     * This function create a regular expression to match all expression in a condition. <br />
     * The following string is created by this function <b>'/%[0-9]+%|#[0-9]+#|\+[0-9]+\+|Q[0-9]+([^\[|0-9]|$)|Q[0-9]+\[[0-9]+\]|~.*?~'</b><br />
     * It matches all expression in a condition and is divided into the following parts:
     *
     * <pre>
     * Qn        /Q[0-9]+(?!\\[)/
     * Qn[m]     /Q[0-9]+\\[[0-9]+\\]/
     * %n%       /%[0-9]+%/
     * +n+       /\\+[0-9]+\\+/
     * #n#       /#[0-9]+#/
     * ~TEXT~    /~.*?~/				Hier gibt es noch Probleme, wenn im Text ein ~ enthalten ist
     * </pre>
     *
     * @return string
     */
    public function getPattern()
    {
        return
            "/" .
            substr(PercentageResultExpression::$pattern, 1, strlen(PercentageResultExpression::$pattern) - 2) . "|" .
            substr(NumericResultExpression::$pattern, 1, strlen(NumericResultExpression::$pattern) - 2) . "|" .
            substr(NumberOfResultExpression::$pattern, 1, strlen(NumberOfResultExpression::$pattern) - 2) . "|" .
            substr(AnswerOfQuestionExpression::$pattern, 1, strlen(AnswerOfQuestionExpression::$pattern) - 2) . "|" .
            substr(ResultOfAnswerOfQuestionExpression::$pattern, 1, strlen(ResultOfAnswerOfQuestionExpression::$pattern) - 2) . "|" .
            substr(StringResultExpression::$pattern, 1, strlen(StringResultExpression::$pattern) - 2) . "|" .
            substr(MatchingResultExpression::$pattern, 1, strlen(MatchingResultExpression::$pattern) - 2) . "|" .
            substr(OrderingResultExpression::$pattern, 1, strlen(OrderingResultExpression::$pattern) - 2) . "|" .
            substr(ExclusiveResultExpression::$pattern, 1, strlen(ExclusiveResultExpression::$pattern) - 2) . "|" .
            substr(EmptyAnswerExpression::$pattern, 1, strlen(EmptyAnswerExpression::$pattern) - 2) .
            "/";
    }

    /**
     * Private constructor to prevent creating of an object of ExpressionManufacturer
     */
    private function __construct()
    {
        require_once __DIR__ . "/../Expressions/AnswerOfQuestionExpression.php";
        require_once __DIR__ . "/../Expressions/ResultOfAnswerOfQuestionExpression.php";
        require_once __DIR__ . "/../Expressions/PercentageResultExpression.php";
        require_once __DIR__ . "/../Expressions/NumberOfResultExpression.php";
        require_once __DIR__ . "/../Expressions/NumericResultExpression.php";
        require_once __DIR__ . "/../Expressions/StringResultExpression.php";
        require_once __DIR__ . "/../Expressions/MatchingResultExpression.php";
        require_once __DIR__ . "/../Expressions/OrderingResultExpression.php";
        require_once __DIR__ . "/../Expressions/ExclusiveResultExpression.php";
        require_once __DIR__ . "/../Expressions/EmptyAnswerExpression.php";
        require_once __DIR__ . '/../Exception/UnsupportedExpression.php';
    }

    /**
     * Private clone to prevent cloning an object of ExpressionManufacturer
     */
    private function __clone()
    {
    }
}
